package Mammoth.AI.NPC.SCXML;

import java.util.HashMap;

import Mammoth.AI.NPC.NPCEvent;
import Mammoth.AI.NPC.Role;
import Mammoth.PhysicsEngine.PhysicsEngine;
import Mammoth.WorldManager.WorldManager;

public class SCXMLEnergySensor extends SCXMLTask {

	private static final String DEFAULT_SCXML = "SCXMLEnergySensor.scxml";

	private HashMap<String, Object> context;
	
	private long lowThreshold;
	private long veryLowThreshold;
	private boolean alive;
	
	//default constructor
	public SCXMLEnergySensor(Role npc) {
		this(npc,10000,2000);
	}
	
	public SCXMLEnergySensor(Role npc, long lowThreshold, long veryLowThreshold) {
		
		super(npc, DEFAULT_SCXML);		

		this.lowThreshold = lowThreshold;
		this.veryLowThreshold = veryLowThreshold;
	}
	
	//create a new SCXMLEnergySensor with external parameters
	public SCXMLEnergySensor(Role npc, String scxmlFile, HashMap<String, Object> params) {		
		super(npc, scxmlFile, params);
	}
	
	public void init() {
		context = new HashMap<String, Object>();
		context.put("this", this);
		
		alive = true;

		this.setContext(context);
		this.startExecution();
	}
	
	public boolean isEnergyHigh() {
		if (npc.getPlayer().getEnergy() >= lowThreshold ) {
			return true;
		}
		return false;
	}
	
	public boolean isEnergyLow() {
		if (npc.getPlayer().getEnergy() < lowThreshold && npc.getPlayer().getEnergy() >= veryLowThreshold) {
			return true;
		}
		return false;
	}
	
	public boolean isEnergyVeryLow() {
		if (npc.getPlayer().getEnergy() < veryLowThreshold ) {
			return true;
		}
		return false;
	}
	
	public void update(long timestep, WorldManager world, PhysicsEngine physics) {
		//fire null event to ensure conditions are checked.
		this.fireEvent(new NPCEvent("null"));
		if (alive) {
			if (npc.getPlayer().getEnergy() == 0.0 ) {
				System.out.println(npc.getPlayer().getName() + " has no energy!");
				alive = false;
				npc.receiveGameEvent(new NPCEvent("out_of_energy"));
			}
		}
	}
	
}
