package Mammoth.AI.NPC.SCXML;

import java.util.Collection;
import java.util.Collections;
import java.util.HashMap;
import java.util.Random;
import java.util.Vector;

import Mammoth.AI.NPC.NPCEvent;
import Mammoth.AI.NPC.Role;
import Mammoth.AI.NPC.RoleImpl;
import Mammoth.PhysicsEngine.PhysicsEngine;
import Mammoth.PhysicsEngine.PhysicsEngineConstants;
import Mammoth.PhysicsEngine.Point;
import Mammoth.WorldManager.Player;
import Mammoth.WorldManager.Position;
import Mammoth.WorldManager.StaticObject;
import Mammoth.WorldManager.WorldManager;
import Mammoth.WorldManager.Actions.ActionException;
import Mammoth.WorldManager.Actions.ActionExecutor;
import Mammoth.WorldManager.Actions.SpawnAction;
import Mammoth.WorldManager.World.WorldManagerImpl;

public class SCXMLFleePlanner extends SCXMLTask {

	private static final String DEFAULT_SCXML = "SCXMLFleePlanner.scxml";

	private HashMap<String, Object> context;
	private Vector<Player> currentThreats;
	private boolean clockwise;
	
	private double dist;
	
	private Vector<StaticObject> trees;
	
	//for tree climbing
	private long CLIMB_COOLDOWN = 15000;
	private long timer;
	private boolean upTree;
	
	
	public SCXMLFleePlanner(Role npc) {
		super(npc, DEFAULT_SCXML);
	}
	
	//create a new SCXMLFleeTask with external parameters
	public SCXMLFleePlanner(Role npc, String scxmlFile, HashMap<String, Object> params) {		
		super(npc, scxmlFile, params);
		Random random = new Random();
		this.clockwise = random.nextBoolean();
		
		//build the list of trees
		trees = new Vector<StaticObject>();
		Collection<StaticObject> obstacles = ((WorldManagerImpl)(npc.getWorldManager())).getStaticObjects();
		for (StaticObject object: obstacles) {
			if (object.getObjectTypeName().equals("TreeSmall_new")) {
				trees.add(object);
			}
		}
		upTree=false;
	}
	
	public void init() {		
		context = new HashMap<String, Object>();
		context.put("this", this);

		this.setContext(context);
		this.startExecution();
		dist = 0;

	}

	//entry points called from the Statechart that generate different flee behaviours:
	public void fleeLow() {
		getThreats();
		fleeStraightLine();
	}
	
	public void fleeHigh() {
		getThreats();
		fleeStraightLine();
	}
	
	public void circleFleeLow() {
		getThreats();
		fleeCircle();
	}
	
	public void circleFleeHigh() {
		getThreats();
		fleeCircle();
	}

	public void treeFleeLow() {
		getThreats();
		fleeTree();
	}
	
	public void treeFleeHigh() {
		getThreats();
		fleeTree();
	}
	
	public void getThreats() {
		SCXMLProximityMemorizer prox = (SCXMLProximityMemorizer)((RoleImpl)npc).getTask(SCXMLProximityMemorizer.class.getName());
		if (!prox.highThreatPlayers.isEmpty()) {
			currentThreats = prox.highThreatPlayers;
		}
		else {
			currentThreats = prox.lowThreatPlayers;
		}
	}
	
	//*********
	// Flee behaviours used internally 
	
	private void fleeStraightLine() {
		if (dist == 0) {
			SCXMLProximityMemorizer prox = (SCXMLProximityMemorizer)((RoleImpl)npc).getTask(SCXMLProximityMemorizer.class.getName());
			dist = prox.getLowThreat();
		}
		//lots we could do here.  threats is a vector of all the players within range of
		//the target that we need to run from.  For now, just run away from the first in the list.
		Point working = getStraightLineUnitVector(npc.getPlayer().getPosition().getPoint(),
						currentThreats.elementAt(0).getPosition().getPoint());
		
		//scale to flee distance
		working.setX(working.getX()*dist);
		working.setY(working.getY()*dist);
		
		//translate to npc's location
		working.add(npc.getPlayer().getPosition().getPoint());
		
		Position destination = (Position)npc.getPlayer().getPosition().clone();
		destination.setPosition(working.getX(), working.getY());
		
		if (!npc.getWorldManager().getWorldShape().contains(destination.getPoint())) {
			destination = selectAlternatePath((Position)destination.clone());
		}
		
		this.getEngine().getRootContext().set("destination", destination);
	}
	
	private Point getStraightLineUnitVector(Point myPosition, Point theirPosition) {
		//get difference
		Point result = myPosition.clone();
		result.minus(theirPosition);
		if (result.getX() == 0 && result.getY() == 0) {
			//they are at the same spot.  this probably shouldn't happen.
			result.setX(1);
			System.out.println("Squirrel is at the same space as its flee target.  Shouldn't happen.");
		}

		//make it a unit vector
		double mag = Math.sqrt((result.getX()*result.getX() + result.getY()*result.getY()));
		result.setX(result.getX()/mag);
		result.setY(result.getY()/mag);
		
		return result;
	}
	
	@SuppressWarnings("unchecked")
	private void fleeTree() {
		//find closest tree
		Vector<TreeDistance> treeDistances = new Vector<TreeDistance>();
		for (StaticObject tree: trees) {
			double dist = tree.getPosition().getPoint().distance(npc.getPlayer().getPosition().getPoint());
			treeDistances.add(new TreeDistance(tree, dist));
		}
		
		Collections.sort(treeDistances);
		
		//now choose the tree to run to
		StaticObject target = null;
		for (TreeDistance targetTree: treeDistances) {
			//see if the player is between us and the tree using the cosine rule
			double a = targetTree.tree.getPosition().getPoint().distance(currentThreats.get(0).getPosition().getPoint());
			double b = targetTree.dist;
			if (b < PhysicsEngineConstants.THRESHOLD) {
				target = targetTree.tree;
				createGameEvent("at_tree", null);
				upTree = true;
				break;
			}
			double c = currentThreats.get(0).getPosition().getPoint().distance(npc.getPlayer().getPosition().getPoint());
			//use cosine rule
			double angle = Math.acos((Math.pow(b, 2) + Math.pow(c,2) - Math.pow(a, 2)) / (2*b*c));
			
			if (angle > Math.PI/2) {
				target = targetTree.tree;
				break;
			}
		}
		if (target == null) {
			fleeStraightLine();
		}
		else {
			//run to the closest tree	
			this.getEngine().getRootContext().set("destination", target.getPosition());
		}
	}
	
	public boolean recentlyTreed() {
		return upTree;
	}

	
	private void fleeCircle() {
		if (dist == 0) {
			SCXMLProximityMemorizer prox = (SCXMLProximityMemorizer)((RoleImpl)npc).getTask(SCXMLProximityMemorizer.class.getName());
			dist = prox.getLowThreat();
		}
		
		//get the unit vector away from the threat
		Point working = getStraightLineUnitVector(npc.getPlayer().getPosition().getPoint(),
				currentThreats.elementAt(0).getPosition().getPoint());
		
		double pAngle = getPolarAngleInRads(working.getX(), working.getY()); 
		//add circular movement
		if (clockwise) {
			pAngle -= Math.PI/4;
		}
		else {
			pAngle += Math.PI/4;
		}
		
		//retrieve new x and y
		working.setX(Math.cos(pAngle));
		working.setY(Math.sin(pAngle));
		
		//scale to flee distance
		working.setX(working.getX()*dist);
		working.setY(working.getY()*dist);
		
		//translate to npc's location
		working.add(npc.getPlayer().getPosition().getPoint());
		
		Position destination = (Position)npc.getPlayer().getPosition().clone();
		destination.setPosition(working.getX(), working.getY());
		
		if (!npc.getWorldManager().getWorldShape().contains(destination.getPoint())) {
			destination = selectAlternatePath((Position)destination.clone());
		}
		
		this.getEngine().getRootContext().set("destination", destination);
		
	}
	
	private double getPolarAngleInRads(double x, double y) {
		double pAngle;
		if (x == 0 && y > 0) {
			pAngle = Math.PI;
		}
		else if (x==0 && y<0) {
			pAngle = Math.PI * 1.5;
		}
		else if (x==0 && y==0) {
			pAngle = 0;
		}
		else {
			pAngle = Math.atan(y / x);
		}
		//correct for quandrant
		//qII
		if (x < 0 && y >= 0) {
			pAngle = Math.PI + pAngle;
		}
		//qIII
		else if (x< 0 && y < 0 ) {
			pAngle = Math.PI + pAngle;
		}
		//qIV
		else if (x >= 0 && y <0) {
			pAngle = 2* Math.PI + pAngle;
		}
		
		return pAngle;
	}
	
	private Position selectAlternatePath(Position pos) {
		
		System.out.println("alternate path");
		
		double posX = pos.getPoint().getX();
		double posY = pos.getPoint().getY();
		
		//figure out why destination is out of bounds
		//check for out of bounds in x, adjust, and update reference value
		double maxX = npc.getWorldManager().getWorldShape().getMaxX();
		double minX = npc.getWorldManager().getWorldShape().getMinX();
		if (posX > maxX) {
			pos.setPosition(Math.min(2* maxX - posX, maxX-0.25), posY);
			posX = pos.getPoint().getX();
		}
		else if (posX < minX) {
			pos.setPosition(Math.max(2* minX - posX, minX+0.25), posY);
			posX = pos.getPoint().getX();
		}
		
		double maxY = npc.getWorldManager().getWorldShape().getMaxY();
		double minY = npc.getWorldManager().getWorldShape().getMinY();
		
		if (posY > maxY) {
			pos.setPosition(posX, Math.min(2* maxY - posY, maxY-0.25));
		}
		else if (posY < minY) {
			pos.setPosition(posX, Math.max(2* minY - posY, minY+0.25));
		}
		return pos;
		
	}
	
	public void update(long timestep, WorldManager world, PhysicsEngine physics) {

		if (upTree) {
			timer+=timestep;
			if (timer > CLIMB_COOLDOWN) {
				upTree = false;
				this.fireEvent(new NPCEvent("timer_up"));
			}
		}
	}
	
	/**
	 * Helper class to simplify finding the closest tree
	 * @author Chris
	 *
	 */
	@SuppressWarnings("rawtypes")
	private class TreeDistance implements Comparable {
		
		public StaticObject tree;
		public double dist;

		public TreeDistance (StaticObject tree, double dist) {
			this.tree = tree;
			this.dist = dist;
		}

		@Override
		public int compareTo(Object arg0) {
			TreeDistance otherTree = (TreeDistance)arg0;
			if (dist < otherTree.dist) return -1;
			else if (dist > otherTree.dist) return 1;
			else return 0;
		}
	}
}
