package Mammoth.AI.NPC.SCXML;

import java.util.HashMap;
import java.util.Vector;

import Mammoth.AI.NPC.Role;
import Mammoth.PhysicsEngine.PhysicsEngine;
import Mammoth.WorldManager.ItemObject;
import Mammoth.WorldManager.WorldManager;

public class SCXMLItemMemorizer extends SCXMLTask {
	
	private static final String DEFAULT_SCXML = "SCXMLItemMemorizer.scxml";
	
	private Vector<ItemObject> items;
	private Vector<ItemObject> keyItems;
	
	private HashMap<String, Object> context;
	
	private String keyItem;

	public SCXMLItemMemorizer(Role npc) {
		this(npc, DEFAULT_SCXML, "Acorn");
	}
	
	public SCXMLItemMemorizer(Role npc, String scxmlFile, String keyItem) {
		super(npc, scxmlFile);
		this.keyItem = keyItem;
	}
	
	
	//create a new SCXMLItemTracker with external parameters
	public SCXMLItemMemorizer(Role npc, String scxmlFile, HashMap<String, Object> params) {		
		super(npc, scxmlFile, params);
	}
	

	public void init() {
		items = new Vector<ItemObject>();
		keyItems = new Vector<ItemObject>();

		context = new HashMap<String, Object>();

		context.put("this", this);
		context.put("key_items", keyItems);
		context.put("items", items);

		this.setContext(context);
		this.startExecution();
	}
	
	public void addItem(ItemObject item) {
		
		
		items.add(item);
		
		if (item.getObjectTypeName().equals(keyItem)) {
			if (keyItems.size() == 0) {
				this.createAIEvent("key_item_visible", "");
			}
			keyItems.add(item);
		}
	}

	public void removeItem(ItemObject item) {
		items.remove(item);
		
		if (item.getObjectTypeName().equals(keyItem)) {
			keyItems.remove(item);
			if (keyItems.size() == 0) {
				this.createAIEvent("no_key_items_visible", "");
			}
		}
	}
	
	public Vector<ItemObject> getKeyItemList() {
		return keyItems;
	}
	

	public void update(long timestep, WorldManager world, PhysicsEngine physics) {
		// nothing to update
	}
	
}
