package Mammoth.AI.NPC.SCXML;

import java.util.HashMap;

import Mammoth.AI.NPC.NPCEvent;
import Mammoth.AI.NPC.Role;
import Mammoth.AI.PathFinding.PathFindingManager.PathType;
import Mammoth.PhysicsEngine.PhysicsEngine;
import Mammoth.WorldManager.Position;
import Mammoth.WorldManager.WorldManager;
import Mammoth.WorldManager.Actions.ActionException;
import Mammoth.WorldManager.Actions.ActionExecutor;
import Mammoth.WorldManager.Actions.MoveAction;

import com.jme3.animation.LoopMode;

public class SCXMLMoveActuator extends SCXMLTask {

	private static final String DEFAULT_SCXML = "SCXMLMoveActuator.scxml";
	boolean moved = false;
	Position destination;

	public SCXMLMoveActuator(Role npc) {
		super(npc, DEFAULT_SCXML);
	}

	// create a new WanderTask with external parameters
	public SCXMLMoveActuator(Role npc, String scxmlFile,
			HashMap<String, Object> params) {
		super(npc, scxmlFile, params);
	}

	public void init() {

		HashMap<String, Object> context = new HashMap<String, Object>();
		context.put("this", this);

		this.setContext(context);
		this.startExecution();
	}

	public void move(Position dest) {
		destination = (Position) dest.clone();

		// create the action and execute it using default path type
		try {
			ActionExecutor.executeSingleAction(new MoveAction(npc.getPlayer(),
					destination.getPoint().getX(), destination.getPoint()
							.getY(), PathType.DIRECT_LINE));
			moved = true;
			npc.playAnimation("run", LoopMode.Loop, (float)(npc.getPlayer().getSpeed() * 1000));

		} catch (ActionException e) {		
			this.createGameEvent("destination_unreachable", null);
		}
	}

	public void stopMove() {
		move(npc.getPlayer().getPosition());
		npc.playAnimation("stand", LoopMode.Loop, 1.0f);
	}

	public void pathMove(Position dest) {
		destination = (Position) dest.clone();

		// create the action and execute it using default path type
		try {
			ActionExecutor.executeSingleAction(new MoveAction(npc.getPlayer(),
					destination.getPoint().getX(), destination.getPoint()
							.getY(), PathType.PATH_FIND));
			moved = true;
			npc.playAnimation("run", LoopMode.Loop, (float)(npc.getPlayer().getSpeed() * 1000));
		} catch (ActionException e) {
			this.createGameEvent("destination_unreachable", null);
		}
	}

	public boolean doneMoving() {
		if (npc.getPlayer().getPath() == null) {
			return !npc.getPlayer().isMoving();
		}
		return false;
	}

	public boolean hasCollided() {
		return (npc.getPlayer().hasCollided());
	}

	public void update(long timestep, WorldManager world, PhysicsEngine physics) {
		if (moved && doneMoving()) {
			if (hasCollided()) {
				this.fireEvent(new NPCEvent("collided", null));
				moved = false;
			}
			if (npc.getPlayer().getPosition().equals(destination)) {
				this.fireEvent(new NPCEvent("destination_reached", null));
			} else {
				this.fireEvent(new NPCEvent("destination_unreachable", null));
			}
			moved = false;
			npc.playAnimation("stand", LoopMode.Loop, 1.0f);
		}
	}

}
