package Mammoth.AI.NPC.SCXML;

import java.util.HashMap;
import java.util.Iterator;
import java.util.Vector;

import Mammoth.AI.NPC.NPCEvent;
import Mammoth.AI.NPC.Role;
import Mammoth.PhysicsEngine.PhysicsEngine;
import Mammoth.PhysicsEngine.Point;
import Mammoth.WorldManager.Player;
import Mammoth.WorldManager.WorldManager;
import Mammoth.WorldManager.Actions.ActionException;
import Mammoth.WorldManager.Actions.ActionExecutor;
import Mammoth.WorldManager.Actions.SetSpeedAction;

public class SCXMLProximityMemorizer extends SCXMLTask {

	private static final String DEFAULT_SCXML = "SCXMLProximityMemorizer.scxml";
	
	private double lowThreat;
	private double highThreat;

	// Three lists of players seen by the AI, sorted by threat level
	Vector<Player> players;
	Vector<Player> lowThreatPlayers;
	Vector<Player> highThreatPlayers;
	private Point npcPosition;

	HashMap<String, Object> context;

	/*
	 * create a new proximity tracker with default values
	 */
	public SCXMLProximityMemorizer(Role npc) {
		this(npc, DEFAULT_SCXML, 1.0, 0.5);
	}
	
	/*
	 * The complete constructor
	 */
	public SCXMLProximityMemorizer(Role npc, String scxmlFile, double low_threat, double high_threat) {
		super(npc, scxmlFile);
		this.lowThreat = low_threat;
		this.highThreat = high_threat;
	}
	
	//create a new SCXMLProximityTracker with external parameters
	public SCXMLProximityMemorizer(Role npc, String scxmlFile, HashMap<String, Object> params) {		
		super(npc, scxmlFile, params);
	}
	
	public void init() {
		players = new Vector<Player>();
		lowThreatPlayers = new Vector<Player>();
		highThreatPlayers = new Vector<Player>();

		context = new HashMap<String, Object>();

		context.put("this", this);
		context.put("lowThreatPlayers", lowThreatPlayers);
		context.put("highThreatPlayers", highThreatPlayers);

		this.setContext(context);
		this.startExecution();	
	}

	public void addPlayer(Player p) {
		if (p.getObjectTypeName().equals("Player")) {
			players.add(p);
		}
	}

	public void removePlayer(Player p) {
		players.remove(p);
	}

	public void update(long timestep, WorldManager world, PhysicsEngine physics) {

		if (highThreatPlayers.isEmpty() && lowThreatPlayers.isEmpty() && npc.getPlayer().getSpeed() != 0.0005) {
			try {
				ActionExecutor.executeSingleAction(SetSpeedAction.createScaledSpeedAction(npc.getPlayer(), 0.5));
			} catch (ActionException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
		}
		
		if (!highThreatPlayers.isEmpty() || !lowThreatPlayers.isEmpty() && npc.getPlayer().getSpeed() != 0.0015) {
			try {
				ActionExecutor.executeSingleAction(SetSpeedAction.createScaledSpeedAction(npc.getPlayer(), 1.5));
			} catch (ActionException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
		}
		
		Player p;
		Point playerPoint = null;
		double dist = 0;
		npcPosition = npc.getPlayer().getPosition().getPoint();

		for (Iterator<Player> i = players.iterator(); i.hasNext();) {
			p = i.next();
			playerPoint = p.getPosition().getPoint();
			dist = npcPosition.distance(playerPoint);

			// determine the threat level of the player and put in the
			// appropriate collection
			if (dist < highThreat) {
				lowThreatPlayers.remove(p);
				if (!highThreatPlayers.contains(p)) {
					highThreatPlayers.add(p);
				}
			} else if (dist < lowThreat) {
				if (!lowThreatPlayers.contains(p)) {
					lowThreatPlayers.add(p);
				}
				highThreatPlayers.remove(p);
			} else {
				lowThreatPlayers.remove(p);
				highThreatPlayers.remove(p);
			}
		}

		// need to fire a dummy event to force the state machine to check its
		// conditions.
		this.fireEvent(new NPCEvent("null"));
	}
	
	public double getLowThreat() {
		return lowThreat;
	}

	public double getHighThreat() {
		return highThreat;
	}

}
