package Mammoth.AI.NPC.SCXML;

import java.util.HashMap;
import java.util.Iterator;
import java.util.Vector;

import Mammoth.AI.NPC.NPCEvent;
import Mammoth.AI.NPC.Role;
import Mammoth.PhysicsEngine.PhysicsEngine;
import Mammoth.PhysicsEngine.Point;
import Mammoth.WorldManager.Player;
import Mammoth.WorldManager.WorldManager;

public class SCXMLSquirrelAnalyzer extends SCXMLTask {

	private static final String DEFAULT_SCXML = "SCXMLSquirrelAnalyzer.scxml";

	// Three lists of players seen by the AI, sorted by threat level
	Vector<Player> players;
	@SuppressWarnings("unused")
	private Point npcPosition;
	private boolean filterEnabled;
	private String filterType;

	HashMap<String, Object> context;

	/*
	 * create a new proximity tracker with default values
	 */
	public SCXMLSquirrelAnalyzer(Role npc, String playerType) {
		this(npc, playerType, DEFAULT_SCXML);
	}
	
	/*
	 * The complete constructor
	 */
	public SCXMLSquirrelAnalyzer(Role npc, String playerType, String scxmlFile) {
		super(npc, scxmlFile);
		if (playerType != null) {
			filterEnabled = true;
			filterType = playerType;
		}
		else {
			filterEnabled = false;
		}
	}
	
	//create a new SCXMLProximityTracker with external parameters
	public SCXMLSquirrelAnalyzer(Role npc, String scxmlFile, HashMap<String, Object> params) {		
		super(npc, scxmlFile, params);
	}
	
	public void init() {
		players = new Vector<Player>();

		context = new HashMap<String, Object>();
		context.put("this", this);

		this.setContext(context);
		this.startExecution();	
	}

	public void addPlayer(Player p) {
		if (filterEnabled) {
			if (p.getObjectTypeName().equals(filterType)) {
				players.add(p);
			}
		}
		else {
			players.add(p);
		}
	}
	
	public Vector<Player> getPlayers() {
		return players;
	}

	public void removePlayer(Player p) {
		players.remove(p);
	}
	
	public boolean playerIsSquirrel(Player p) {
		if (p.getObjectTypeName().equals(filterType)) {
			return true;
		}
		return false;
	}

	public boolean playersVisible() {
		return !players.isEmpty();
	}
	
	public void update(long timestep, WorldManager world, PhysicsEngine physics) {
		//nothing time based in this task.
	}

}
