package Mammoth.AI.NPC.SCXML;

import java.util.HashMap;
import java.util.LinkedList;

import org.apache.commons.scxml.TriggerEvent;
import org.apache.commons.scxml.model.ModelException;

import Mammoth.AI.NPC.NPCEvent;
import Mammoth.AI.NPC.Role;
import Mammoth.AI.NPC.RoleImpl;
import Mammoth.PhysicsEngine.PhysicsEngine;
import Mammoth.WorldManager.Position;
import Mammoth.WorldManager.WorldManager;

public class SCXMLStutterTask extends SCXMLTask {

	private static final String DEFAULT_SCXML = "SCXMLStutterTask.scxml";
	
	String target;
	Position finalDestination;
	Position currentStep;

	LinkedList<Position> stepQueue;
	
	public SCXMLStutterTask(Role npc, String stutterTarget) {
		super(npc, DEFAULT_SCXML);
		
		target=stutterTarget;
		
		((RoleImpl)npc).addEventFilter(target, "move", "stutter_move");
	}
	
	public void init(){
		stepQueue = new LinkedList<Position>();
		
		HashMap<String,Object> context = new HashMap<String,Object>();
		context.put("this", this);
		this.setContext(context);
		this.startExecution();
	}
	
	//create a new SCXMLStutterTask with external parameters
	public SCXMLStutterTask(Role npc, String scxmlFile, HashMap<String, Object> params) {		
		super(npc, scxmlFile, params);
		
		((RoleImpl)npc).addEventFilter(target, "move", "stutter_move");
	}
	
	public void newDestination(Object destination) {
		finalDestination = (Position)((Position)destination).clone();
		stepQueue.clear();
		
		//now determine our random steps
		double dist = 0;
		while (dist < 1) {

			//decide a random percent of the path to move
			dist += (Math.random()*0.5) + 0.1;
			if (dist>=1) {
				stepQueue.add(finalDestination);
			}
			else {
				//calculate the next position
				Position current = npc.getPlayer().getPosition();
				Position target = (Position)finalDestination.clone();
				
				target.setPosition((finalDestination.getPoint().getX() - current.getPoint().getX()) * dist + current.getPoint().getX(),
					(finalDestination.getPoint().getY() - current.getPoint().getY()) * dist + current.getPoint().getY());
				
				//queue it up
				stepQueue.add(target);
			}
		}
	}
	
	public void nextStep() {
		if (!stepQueue.isEmpty()) {
			currentStep = stepQueue.poll();
			npc.receiveAIEvent(new NPCEvent("move", currentStep));
		}
		else {
			npc.receiveAIEvent(new NPCEvent("final_destination_reached", currentStep));
		}
	}

	public void update(long timestep, WorldManager world, PhysicsEngine physics) {
		
		try {
			this.getEngine().triggerEvent(new TriggerEvent("time",  TriggerEvent.SIGNAL_EVENT, ((Long)timestep).toString()));
		} catch (ModelException e) {
			
			e.printStackTrace();
		}
		
	}

}
