package Mammoth.AI.NPC.SCXML;

import java.util.HashMap;

import org.apache.commons.scxml.TriggerEvent;
import org.apache.commons.scxml.model.ModelException;

import Mammoth.AI.NPC.NPCEvent;
import Mammoth.AI.NPC.Role;
import Mammoth.AI.NPC.RoleImpl;
import Mammoth.PhysicsEngine.PhysicsEngine;
import Mammoth.Util.Collection.CollectionUtils;
import Mammoth.WorldManager.Position;
import Mammoth.WorldManager.WorldManager;

public class SCXMLWanderPlanner extends SCXMLTask {

	private static final String DEFAULT_SCXML = "SCXMLWanderPlanner.scxml";
	
	//the x,y range that the npc can wander in each step
	private double xRadius;
	private double yRadius;
	
	//the minimum rest time, and the range
	private int restTimeMin;
	private int restTimeRange;
	
	//create a new WanderTask with default settings
	public SCXMLWanderPlanner(Role npc) {
		this(npc, DEFAULT_SCXML, 2.5, 2.5, 2000, 5000 );
	}
	
	//complete constructor
	public SCXMLWanderPlanner(Role npc, String scxmlFile, double xRad, double yRad, int restMin, int restRange) {
		super(npc, scxmlFile);
		
		xRadius = xRad;
		yRadius = yRad;
		restTimeMin=restMin;
		restTimeRange=restRange;
	}
	
	//create a new WanderTask with external parameters
	public SCXMLWanderPlanner(Role npc, String scxmlFile, HashMap<String, Object> params) {		
		super(npc, scxmlFile, params);
	}
	
	// begin executing the state-chart
	public void init() {
		HashMap<String,Object> context = new HashMap<String,Object>();
		context.put("this", this);
		context.put("restTime", (int)(Math.random()* restTimeRange) + restTimeMin);
		this.setContext(context);
		this.startExecution();
	}
	
	public void newRandomDestination() {
		Position nextPos;
		// do loop - repeats until a location in the world has been chosen.  While this is potentially an infinite loop, 
		// the mammoth world is square, so worst case is an NPC is in a corner, which essentially means a 25% chance of
		// randoming an in-bounds wander destination.
		
		do {
		Position curPos = ((RoleImpl)npc).getPosition();
		double x = curPos.getPoint().getX();
		double y = curPos.getPoint().getY();
		nextPos = (Position)curPos.clone();
		double delta_x = CollectionUtils.random.nextDouble()*(2*xRadius) - xRadius; 
		double delta_y = CollectionUtils.random.nextDouble()*(2*yRadius) - yRadius;
		
		nextPos.setPosition(x + delta_x, y + delta_y);
		
		} while (!npc.getWorldManager().getWorldShape().contains(nextPos.getPoint().getX(), nextPos.getPoint().getY()));
		
		npc.receiveAIEvent(new NPCEvent("move", nextPos));	
	}

	public void update(long timestep, WorldManager world, PhysicsEngine physics) {
		try {
			this.getEngine().triggerEvent(new TriggerEvent("time",  TriggerEvent.SIGNAL_EVENT, ((Long)timestep).toString()));
		} catch (ModelException e) {
			
			e.printStackTrace();
		}
	}
	
}
